<?php

namespace WPCWV;

use WP_Rocket\Engine\Cache\AdvancedCache;

if (!defined('ABSPATH')) {
    die('Let\'s not do this');
}
?>

<div class="cwv-tab-panel" id="tabs-beta" style="display: none;">

    <h2 class="cwv-tab-heading">
        <svg class="icon">
            <use xlink:href="#icon-beta"></use>
        </svg>
        Beta Features
        <svg data-help=".betahelp" class="cwv-help helpicon">
            <use xlink:href="#icon-question"></use>
        </svg>
    </h2>

    <div class="cwvhelptext betahelp">
        <p>Beta features, use at your own risk!!.</p>

    </div>

    <table class="form-table">
        <tr valign="top">
            <th scope="row">Show beta features</th>
            <td>
                <div class="switch-field">
                    <input name="cwv_show_beta_features" id="cwv_show_beta_features0" type="radio" value="0" <?php checked('0', cwv_options::get('cwv_show_beta_features')); ?> />
                    <label for="cwv_show_beta_features0">no</label>
                    <input name="cwv_show_beta_features" id="cwv_show_beta_features1" type="radio" value="1" <?php checked('1', cwv_options::get('cwv_show_beta_features')); ?> />
                    <label for="cwv_show_beta_features1">yes</label>
                </div>
                <p class="inlinehelp">Show beta features? Use at your own risk! Current beta features are 'Modern Image Formats: AVIF And WebP'</p>
            </td>
        </tr>
    </table>

    <div data-depends="cwv_show_beta_features">
        <div></div>
        <h2 class="cwv-tab-heading">
            <svg class="icon">
                <use xlink:href="#icon-cache"></use>
            </svg>
            Activate ultra fast cache
        </h2>
    </div>

    <div data-depends="cwv_show_beta_features">
        <table class="form-table">
            <tr valign="top">
                <th scope="row">Enable direct WP Adavanced Cache</th>
                <td>
                    <div class="switch-field">
                        <input name="cwv_use_advanced_cache" id="cwv_use_advanced_cache0" type="radio" value="0" <?php checked('0', cwv_options::get('cwv_use_advanced_cache')); ?> />
                        <label for="cwv_use_advanced_cache0">no</label>
                        <input name="cwv_use_advanced_cache" id="cwv_use_advanced_cache1" type="radio" value="1" <?php checked('1', cwv_options::get('cwv_use_advanced_cache')); ?> />
                        <label for="cwv_use_advanced_cache1">yes</label>
                    </div>
                    <p class="inlinehelp">WP advanced Caches uses a much faster caching mechanism but requires simple manual setup. </p>
                </td>
            </tr>
        </table>

        
        <?php if (false == defined('WP_CACHE') &&  false == FileSystem::file_is_writable('wp-config')) { ?>
            <div data-depends="cwv_use_advanced_cache">
                
                <p><b>WP-content not writable</b> We cannot automatically setup advanced cache for you because <code>wp-config.php</code> is not writable. Please make sure the file is writable or manually add '<code>define( 'WP_CACHE', true );</code>' to this file.</p>
            </div>
        <?php } ?>

        <?php if (false == FileSystem::file_is_writable('advanced-cache')) { ?>
            <div data-depends="cwv_use_advanced_cache">

                
                <p><b>Advanced-cache.php not writable:</b> We cannot automatically setup advanced cache for you because <code>wp-content/advanced-cache.php</code> is not writable.
                    Please make sure the file is writable or manually overwrite the file with this code:<br />
                <pre><?php echo htmlentities(cwv_advanced_cache::$advancedCacheCode); ?></pre>
                </p>
            </div>
        <?php } ?>


    </div>


    <div data-depends="cwv_show_beta_features">
        <div></div>
        <h2 class="cwv-tab-heading">
            <svg class="icon">
                <use xlink:href="#icon-images"></use>
            </svg>
            Modern Image Formats: AVIF And WebP
        </h2>
        <p>WP Core Web Vitals can convert images on the fly, cache them and serve modern, faster image formats (AVIF and WebP). Currently the Converter has been tested with NGINX.</p>
        <img width="1391" height="487" loading="lazy" decoding="async" src="<?php echo WP_CWV_PLUGIN_DIR; ?>/admin/img/jpg-web-avif.png">
        <h3>Getting started</h3>
        <ol>
            <li>Make sure the cache dir writeable? <b>hint: it is <?php echo FileSystem::dir_is_writable('cachedir') ? '!' : 'not :-('; ?></b></li>
            <li>Check if you are running an APACHE or NGINX web server. If you are unsure stop now and ask your hosting provider or server admin.</li>
            <li>Add the config to your .htaccess file (for APACHE Webservers) or you nginx config file (for NGINX servers) and test it!</li>
        </ol>

        <h3>NGINX config</h3>
        <p>Ask your system administrator to add the following rules to the NGINX config!</p>
        <pre>set $cwv_ext_webp ".webp";
set $cwv_ext_avif ".avif";
set $cwv_path_webp "/wp-content/cache/cwv_img/";
set $cwv_path_avif "/wp-content/cache/cwv_img/";

if ($http_accept !~* "image/webp") {
        set $cwv_ext_webp "";
        set $cwv_path_webp "/wp-content/";
}


if ($http_accept !~* "image/avif") {
        set $cwv_ext_avif "";
        set $cwv_path_avif "/wp-content/";
}

location ~ /wp-content/(?<path>.+)\.(?<ext>jpe?g|png)$ {
        add_header Vary Accept;
        expires 365d;
        try_files
                $cwv_path_avif$path.$ext$cwv_ext_avif
                $cwv_path_webp$path.$ext$cwv_ext_webp
                /wp-content/plugins/wp-core-web-vitals/converter/convert.php;
}</pre>


        <h3>APACHE config</h3>
        <p>Add the following instruction to your .htaccess!</p>
        <pre><?php echo htmlentities('<IfModule mod_rewrite.c>
  RewriteEngine On

  # Try AVIF image
  RewriteCond %{HTTP_ACCEPT} image/avif
  RewriteCond %{REQUEST_URI} ^/wp-content/(.*)\\.(jpe?g|png)$
  RewriteCond %{DOCUMENT_ROOT}/wp-content/cache/cwv_img/$1.$2.avif -f
  RewriteRule ^wp-content/(.*)\\.(jpe?g|png)$ /wp-content/cache/cwv_img/$1.$2.avif [T=image/avif,E=REQUEST_image,L]

  # Try WebP Image
  RewriteCond %{HTTP_ACCEPT} image/webp
  RewriteCond %{REQUEST_URI} ^/wp-content/(.*)\\.(jpe?g|png)$
  RewriteCond %{DOCUMENT_ROOT}/wp-content/cache/cwv_img/$1.$2.webp -f
  RewriteRule ^wp-content/(.*)\\.(jpe?g|png)$ /wp-content/cache/cwv_img/$1.$2.webp [T=image/webp,E=REQUEST_image,L]

  # Rewrite to the Converter
  RewriteCond %{HTTP_ACCEPT} image/webp
  RewriteCond %{REQUEST_URI} ^/wp-content/(.*)\\.(jpe?g|png)$
  RewriteCond %{DOCUMENT_ROOT}/wp-content/cache/cwv_img/$1.$2.(webp|avif) !-f
  RewriteRule ^wp-content/(.*)\\.(jpe?g|png)$ /wp-content/plugins/wp-core-web-vitals/converter/convert.php [QSA,L]
</IfModule>

<IfModule mod_setenvif.c>
  # Create REQUEST_IS_IMAGE variable for image files
  SetEnvIf Request_URI "\\.(png|jpe?g)$" REQUEST_IS_IMAGE
</IfModule>

<IfModule mod_headers.c>
  # Add the Vary header to the response of the image request,
  # saying that the content at this URL may differ depending
  # on whether browser supports WebP or not
  Header set Vary "Accept,Accept-Encoding" env=REQUEST_IS_IMAGE
</IfModule>


<IfModule mod_mime.c>
  AddType image/avif .avif
  AddType image/webp .webp
</IfModule>', ENT_COMPAT); ?></pre>



        <div id="fontfiles"></div>

    </div>




</div>