<?php

namespace WPCWV;

if (!defined('ABSPATH')) {
    die('Let\'s not do this');
}
?>

<script src="https://cdn.jsdelivr.net/npm/chart.js"></script>
<script src="https://cdn.jsdelivr.net/npm/chartjs-adapter-date-fns/dist/chartjs-adapter-date-fns.bundle.min.js"></script>

<div class="cwv-tab-panel" id="tab-rum" style="display: none;">
    <h2 class="cwv-tab-heading">
        <svg class="icon">
            <use xlink:href="#icon-css3"></use>
        </svg>
        PageSpeed Monitoring
        <svg data-help=".rumhelp" class="cwv-help helpicon">
            <use xlink:href="#icon-question"></use>
        </svg>
    </h2>


    <div class="cwvhelptext rumhelp">
        <p>PageSpeed monitoring is a great way to keep track of the performance of your website.
            It measures the Real User Metrics (RUM) of your website (allmost) idential on how Google measures the Core Web Vitals.
            It will give you a good overview of the <a href="https://www.corewebvitals.io">Core Web Vitals</a> of your site without affecting performance.
            It will also give you a good overview of the performance of your site over time.
    </div>


    <table class="form-table" width="100%">
        <tr valign="top">
            <th scope="row">RUM Tracking</th>
            <td>
                <div class="switch-field">
                    <input name="wp_cwv_collect_rum" id="wp_cwv_collect_rum0" type="radio" value="0" <?php checked('0', cwv_options::get('wp_cwv_collect_rum')); ?> /><label for="wp_cwv_collect_rum0">no</label>
                    <input name="wp_cwv_collect_rum" id="wp_cwv_collect_rum1" type="radio" value="1" <?php checked('1', cwv_options::get('wp_cwv_collect_rum')); ?> /><label for="wp_cwv_collect_rum1">yes</label>
                </div>
            </td>
        </tr>
        <tr valign="top" data-depends="wp_cwv_collect_rum">
            <th scope="row">Sample rate</th>
            <td>
                <select name="cwv_rum_sample_rate">
                    <option value="1" <?php selected('1', cwv_options::get('cwv_rum_sample_rate')); ?>>1%</option>
                    <option value="2" <?php selected('2', cwv_options::get('cwv_rum_sample_rate')); ?>>2%</option>
                    <option value="5" <?php selected('5', cwv_options::get('cwv_rum_sample_rate')); ?>>5%</option>
                    <option value="10" <?php selected('10', cwv_options::get('cwv_rum_sample_rate')); ?>>10%</option>
                    <option value="20" <?php selected('20', cwv_options::get('cwv_rum_sample_rate')); ?>>20%</option>
                    <option value="30" <?php selected('30', cwv_options::get('cwv_rum_sample_rate')); ?>>30%</option>
                    <option value="40" <?php selected('40', cwv_options::get('cwv_rum_sample_rate')); ?>>40%</option>
                    <option value="50" <?php selected('50', cwv_options::get('cwv_rum_sample_rate')); ?>>50%</option>
                    <option value="60" <?php selected('60', cwv_options::get('cwv_rum_sample_rate')); ?>>60%</option>
                    <option value="70" <?php selected('70', cwv_options::get('cwv_rum_sample_rate')); ?>>70%</option>
                    <option value="80" <?php selected('80', cwv_options::get('cwv_rum_sample_rate')); ?>>80%</option>
                    <option value="90" <?php selected('90', cwv_options::get('cwv_rum_sample_rate')); ?>>90%</option>
                    <option value="100" <?php selected('100', cwv_options::get('cwv_rum_sample_rate')); ?>>100%</option>
                </select>
                <p class="inlinehelp">
                    <b>Important</b>, adjust the sample rate to fetch about 200/500 samples per day.
                    The formula is really easy.
                    If you have 1000 visitors per day, you can set the sample rate to 20% to get 200 samples per day.
                    This will give you a good overview of the <a href="https://www.corewebvitals.io">Core Web Vitals</a> of your site without affecting performance.
                </p>
            </td>
        </tr>


        <tr valign="top" data-depends="wp_cwv_collect_rum">
            <th scope="row">Retention</th>
            <td>
                <select name="cwv_rum_sample_retention">
                    <option value="7" <?php selected('7', cwv_options::get('cwv_rum_sample_retention')); ?>>7 days</option>
                    <option value="14" <?php selected('14', cwv_options::get('cwv_rum_sample_retention')); ?>>14 days</option>
                    <option value="30" <?php selected('30', cwv_options::get('cwv_rum_sample_retention')); ?>>30 days</option>
                    <option value="60" <?php selected('60', cwv_options::get('cwv_rum_sample_retention')); ?>>60 days</option>
                    <option value="90" <?php selected('90', cwv_options::get('cwv_rum_sample_retention')); ?>>90 days</option>
                </select>
            </td>

        </tr>
        <tr valign="top" data-depends="wp_cwv_collect_rum">
            <th scope="row">Reset data</th>
            <td>
                <a id="cwv_truncate_rum" class="button button-primary" href="#">Reset RUM data</a>
                <a id="cwv_tmp_to_main" class="button button-primary" href="#">Refresh RUM data</a>

            </td>
            <td></td>

        </tr>
    </table>



    <div data-depends="wp_cwv_collect_rum">
        <div></div>
        <h2 class="cwv-tab-heading">
            <svg class="icon">
                <use xlink:href="#icon-css3"></use>
            </svg>
            RUM data
            <svg data-help=".rumcharthelp" class="cwv-help helpicon">
                <use xlink:href="#icon-question"></use>
            </svg>
        </h2>


        <div class="cwvhelptext rumcharthelp">
            <p>The RUM Data charts show the Real User Metrics for the Largest Contentful Paint, First Input Delay, Cumulative Layout Shift and First Contentful Paint.
                The charts show the distribution of the metrics over time.
                The charts are based on the data collected by the PageSpeed monitoring.
                The charts are updated every hour or when you refresh the RUM data manually</p>
        </div>


        <div class="rum_filter">
            <div class="rum_filter_option rum_device active" data-device="all">ALL</div>
            <div class="rum_filter_option rum_device" data-device="desktop">Desktop</div>
            <div class="rum_filter_option rum_device" data-device="mobile">Mobile</div>
        </div>

        <div class="rum_filter">
            <div class="rum_filter_option rum_metric active" data-metric="all">ALL</div>
            <div class="rum_filter_option rum_metric" data-metric="LCP">LCP</div>
            <div class="rum_filter_option rum_metric" data-metric="FID">FID</div>
            <div class="rum_filter_option rum_metric" data-metric="CLS">CLS</div>
            <div class="rum_filter_option rum_metric" data-metric="FCP">FCP</div>
            <div class="rum_filter_option rum_metric" data-metric="INP">INP</div>
            <div class="rum_filter_option rum_metric" data-metric="TTFB">TTFB</div>
        </div>

        <div class="cwv-rum-grid">
            

            <div class="cwvcard cwv-rum-card" id="LCP">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="LCP">
                        <h2>LCP</h2>
                        <p>Largest Contentful Paint</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>


            </div>
            <div class="cwvcard cwv-rum-card" id="FID">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="FID">
                        <h2>FID</h2>
                        <p>First Input Delay</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>




            </div>
            <div class="cwvcard cwv-rum-card" id="CLS">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="CLS">
                        <h2>CLS</h2>
                        <p>Cumulative Layout Shift</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>



            </div>
            <div class="cwvcard cwv-rum-card" id="FCP">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="FCP">
                        <h2>FCP</h2>
                        <p>First Contentful Paint</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>



            </div>
            <div class="cwvcard cwv-rum-card" id="INP">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="INP">
                        <h2>INP</h2>
                        <p>Interaction to Next Paint</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>



            </div>
            <div class="cwvcard cwv-rum-card" id="TTFB">
                <div class="cwv-rum-card-header">
                    <div class="cwv-rum-title" data-metric="TTFB">
                        <h2>TTFB</h2>
                        <p>Time to first byte</p>
                    </div>
                    <div class="cwv-rum-value">

                    </div>
                </div>
                <div class="cwv-rum-progress"></div>
                <div class="cwv-rum-timeline"><canvas></canvas></div>
                <div class="cwv-rum-histogram"><canvas></canvas></div>



            </div>
            <div id="urllist" class="cwvcard cwv-rum-card" style="display: none;">

            </div>
        </div>
    </div>
</div>