<?php

namespace WPCWV;

if (!defined('ABSPATH')) {
    die('Let\'s not do this');
}

/** 
 * RUM TRACKING, mostly self contained rum class
 * There is some rum stuff left in the settings class
 * @todo, truncate data after xx days
 **/

class cwv_rum
{

    public static function init()
    {

        if (is_admin() || (function_exists('is_login') && is_login())) {
            self::backend_actions();
        } else {
            // 2 reasons not to track
            if (!defined('DONOTCACHEPAGE')) {
                // do not track admin
                if (!\current_user_can('manage_options')) {
                    self::trackingscript();
                }
            }
        }
    }

    private static function trackingscript()
    {
        $b = cwv_options::get('wp_cwv_collect_rum');

        if ($b) {
            $pct = (int)cwv_options::get('cwv_rum_sample_rate');
            if ($pct <= 0 || $pct > 100) {
                $pct = 100;
            }
            $sample_rate = $pct / 100;
        } else {
            $sample_rate = 0.005;
        }

        $cwv_rum_endpoint = json_encode(admin_url('admin-ajax.php'));

        $script = 'var cwv_rum_endpoint=' . $cwv_rum_endpoint . ';' . $sample_rate . '>Math.random()&&window.addEventListener("load",function(){var e=document.createElement("script");e.src="' . WP_CWV_PLUGIN_DIR . 'public/js/rum.js?ver=' . WP_CWV_VERSION . '",document.body.appendChild(e)});';

        wp_register_script('wp_cwv_rum_tracking', '', [], '', true);
        wp_enqueue_script('wp_cwv_rum_tracking');
        wp_add_inline_script('wp_cwv_rum_tracking', $script);
    }


    private static function backend_actions()
    {

        // actions
        add_action('wp_ajax_cwv_rum_cache', 'WPCWV\cwv_rum::admin_ajax_cwv_rum_cache');
        add_action('wp_ajax_cwv_get_progress', 'WPCWV\cwv_rum::admin_ajax_cwv_get_progress');
        add_action('wp_ajax_cwv_get_p75', 'WPCWV\cwv_rum::admin_ajax_cwv_get_p75');
        add_action('wp_ajax_cwv_get_histogram', 'WPCWV\cwv_rum::admin_ajax_cwv_get_histogram');
        add_action('wp_ajax_cwv_get_timeline', 'WPCWV\cwv_rum::admin_ajax_cwv_get_timeline');
        add_action('wp_ajax_cwv_get_urllist', 'WPCWV\cwv_rum::admin_ajax_cwv_get_urllist');

        add_action('wp_ajax_cwv_truncate_rum', 'WPCWV\cwv_rum::admin_ajax_cwv_truncate_rum');
        add_action('wp_ajax_cwv_tmp_to_main', 'WPCWV\cwv_rum::admin_ajax_cwv_tmp_to_main');

        add_action('wp_ajax_collect_cwv_rum', 'WPCWV\cwv_rum::admin_ajax_collect_cwv_rum');
        add_action('wp_ajax_nopriv_collect_cwv_rum', 'WPCWV\cwv_rum::admin_ajax_nopriv_collect_cwv_rum');
    }

    /**
     * precache rum data (does this need to be in the DB RUM class?)
     */
    public static function precache()
    {
        $data = [];
        foreach (RumDB::$aMetrics as $metric) {
            $data[$metric]['timeline'] = RumDB::timeline($metric);
            $data[$metric]['histogram'] = RumDB::histogram($metric);
            $data[$metric]['progress'] = RumDB::progress($metric);
            $p75 = RumDB::percentile($metric, 75);
            $data[$metric]['p75'] = $p75;
        }
        // save to transient
        set_transient('cwv_rum_cache', json_encode($data), 60 * 60 * 24);
    }

    // ajax admin function get rum data from cache
    public static function admin_ajax_cwv_rum_cache()
    {
        // get transient
        $data = get_transient('cwv_rum_cache');
        if (!$data || $_POST['d'] != 'all') {
            $data = json_encode([]);
        }
        echo $data;
        exit();
    }


    // ajax admin function clear all rum data
    public static function admin_ajax_cwv_truncate_rum()
    {
        RumDB::truncate();

        // clear transient
        delete_transient('cwv_rum_cache');

        echo json_encode(['success' => true]);
        exit();
    }

    // ajax admin function get progress for metric
    public static function admin_ajax_cwv_tmp_to_main()
    {
        RumDB::tmp_to_main();
        cwv_rum::precache();
        echo json_encode(['success' => true]);
        exit();
    }

    // ajax admin function get progress for metric
    public static function admin_ajax_cwv_get_progress()
    {
        $metric = $_POST['metric'];
        $bucket = RumDB::progress($metric);
        echo json_encode($bucket);
        exit();
    }

    // ajax admin function get p75 for metric
    public static function admin_ajax_cwv_get_p75()
    {
        $metric = $_POST['metric'];
        $p75 = RumDB::percentile($metric, 75);
        echo json_encode(['p75' => $p75]);
        exit();
    }
    // ajax admin function get p75 for metric
    public static function admin_ajax_cwv_get_histogram()
    {
        $metric = $_POST['metric'];
        $histogram = RumDB::histogram($metric);
        echo json_encode($histogram);
        exit();
    }
    // ajax admin function get p75 for metric for timeline
    public static function admin_ajax_cwv_get_timeline()
    {
        $metric = $_POST['metric'];
        $timeline = RumDB::timeline($metric);
        echo json_encode($timeline);
        exit();
    }


    // ajax admin function get p75 for metric for each url
    public static function admin_ajax_cwv_get_urllist()
    {
        $metric = $_POST['metric'];
        $urllist = RumDB::urllist($metric);
        echo json_encode($urllist);
        exit();
    }


    public static function admin_ajax_collect_cwv_rum()
    {
        echo json_encode(['thank' => 'you']);
        exit();
    }

    /** 
     * save the rum data
     * @todo, save query string!
     **/
    public static function admin_ajax_nopriv_collect_cwv_rum()
    {

        $data = json_decode(stripslashes($_POST['metrics']), true);

        foreach ($data as $d) {
            $id = $d['id'];
            $metricname = $d['n'];
            $metricvalue = $d['v'];
            // remove query string from $d['u']
            $url = strtok($d['u'], '?');
            $device = $d['d'];

            RumDB::upsert($id, $metricname, $metricvalue, $url, $device);

            if ($metricname == 'LCP' && !empty($d['advisor']) && cwv_options::get('wp_cwv_advisor') == 1) {
                $json = json_encode(
                    [
                        'src' => $d['advisor']['src'],
                        'class' => $d['advisor']['class'],
                        'parentclass' => $d['advisor']['parentclass'],
                        'url' => $url
                    ]
                );
                AdvisorDB::insert(['type' => 'LCP', 'json' => $json]);
            }
        }
        echo json_encode(['thank' => 'you']);
        exit();
    }
}
